#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from tornado.options import define
from tornado.options import options as tornado_options

define("port", default=8888, type=int, metavar="PORT", help="main port (of the lock manager)", group="rdlm")

class Options(object):
    '''Class to store command line options'''

    @property
    def port(self):
        '''
        @summary: returns the main port of the daemon
        @result: the port of the daemon (as an integer)
        '''
        return tornado_options.port



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import tornado.testing
from rdlm.main import get_app as rdlm_get_app

class HelloTestCase(tornado.testing.AsyncHTTPTestCase):

    def get_app(self):
        return rdlm_get_app()

    def test_home(self):
        self.http_client.fetch(self.get_url('/'), self.stop)
        response = self.wait()
        self.assertIn("Welcome", response.body)
        self.assertEqual(response.code, 200)
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.abstract_lock_handler import AbstractLockHandler
from rdlm.lock import LOCK_MANAGER_INSTANCE

class LocksHandler(AbstractLockHandler):
    """Class which handles the /locks/[...]/ URL"""

    SUPPORTED_METHODS = ['GET', 'DELETE']

    def get(self, name, uid): # pylint: disable-msg=W0221
        '''
        @summary: deals with GET request
        @param name: name of the resource
        @param uid: uid of the lock
        '''    
        active_lock = LOCK_MANAGER_INSTANCE.get_active_lock(name)
        if active_lock and (active_lock.uid == uid):
            self.set_header('Content-Type', 'application/json')
            self.write(active_lock.to_json())
        else:
            self.send_error(status_code=404)
            return

    def delete(self, name, uid): # pylint: disable-msg=W0221
        '''
        @summary: deals with DELETE request (releasing a lock)
        @param name: name of the resource
        @param uid: uid of the lock
        '''
        active_lock = LOCK_MANAGER_INSTANCE.get_active_lock(name)
        if active_lock and (active_lock.uid == uid):
            LOCK_MANAGER_INSTANCE.remove_active_lock(name)
            self.set_status(204)
            self.finish()
        else:
            self.send_error(status_code=404)
            return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.request_handler import RequestHandler
from rdlm.lock import LOCK_MANAGER_INSTANCE

class UnitTestingHandler(RequestHandler):
    """Class which handles some unit testing handler URL"""

    def get(self):
        '''
        @summary: deals with GET request on /reset
        '''
        LOCK_MANAGER_INSTANCE._reinit() # pylint: disable-msg=W0212


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.request_handler import RequestHandler

class AbstractLockHandler(RequestHandler):
    """Just an abstract class to store some utilities methods"""

    def get_base_url(self, request):
        '''
        @summary: returns the http://hostname:port part of an url
        @param request: incoming HttpRequest object (from tornado)
        @result: the http://hostname:port part of an url

        The base url is built from another incoming http request
        to get the correct hostname and port as choosen by the 
        client (in order to avoid some problems with proxy 
        configurations)
        '''
        return "%s://%s" % (request.protocol, request.host)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.request_handler import RequestHandler

class HelloHandler(RequestHandler):
    """Class which handles the / URL"""

    def get(self):
        '''
        @summary: deals with GET request on /
        '''
        self.write('Welcome on restful-distributed-lock-manager !')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

"""
This file is just a wrapper to start the daemon
"""
import os
import sys

if __name__ == '__main__':
    sys.path.append(os.path.dirname(__file__))
    from rdlm.main import main
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

import tornado.ioloop
import tornado.web

from rdlm.options import Options 
from rdlm.hello_handler import HelloHandler
from rdlm.active_locks_handler import ActiveLocksHandler
from rdlm.locks_handler import LocksHandler
from rdlm.unit_testing import UnitTestingHandler
from rdlm.lock import LOCK_MANAGER_INSTANCE

def on_every_second():
    '''
    @summary: function called by tornado/ioloop every second
    
    It's used to clear expired locks
    '''
    LOCK_MANAGER_INSTANCE.clean_expired_locks()

def get_app(unit_testing = False):
    '''
    @summary: returns the tornado application
    @param unit_testing: if True, we add some handler for unit testing only
    @result: the tornado application
    '''
    url_list = [
        tornado.web.URLSpec(r"/", HelloHandler, name="hello"),
        tornado.web.URLSpec(r"/active_locks/([a-zA-Z0-9]+)", ActiveLocksHandler, name="active_locks"),
        tornado.web.URLSpec(r"/locks/([a-zA-Z0-9]+)/([a-zA-Z0-9]+)", LocksHandler, name="lock")
    ]
    if unit_testing:
        url_list.append(tornado.web.URLSpec(r"/reset", UnitTestingHandler, name="unit_testing"))
    application = tornado.web.Application(url_list)
    return application

def get_ioloop():
    iol = tornado.ioloop.IOLoop.instance()
    tornado.ioloop.PeriodicCallback(on_every_second, 1000, iol).start()
    return iol

def main():
    '''
    @summary: main function (starts the daemon)
    '''
    application = get_app()
    tornado.options.parse_command_line()
    application.listen(Options().port)
    iol = get_ioloop()
    iol.start()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

import uuid
import datetime
import collections
import json

class Lock(object):
    '''
    Class which defines a Lock object

    The lock object has a status :
    - active (the lock is acquired)
    - not active (the lock is not acquired)
    '''

    __active = False
    uid = None
    title = None
    resource_name = None
    lifetime = 0
    wait = 0
    wait_since = None
    wait_expires = None
    active_since = None
    active_expires = None
    __active_callback = None
    __delete_callback = None

    def __init__(self, resource_name, title, wait, lifetime):
        '''
        @summary: lock constructor
        @param resource_name: name of the resource to lock
        @param title: title of the lock
        @param wait: wait max duration (in seconds)
        @param lifetime: timeout of the lock (in seconds)
        @result: lock object (not active) 
        '''
        self.resource_name = resource_name
        self.lifetime = lifetime
        self.title = title
        self.uid = str(uuid.uuid4()).replace('-', '')
        self.wait = wait
        self.lifetime = lifetime
        self.wait_since = datetime.datetime.now()
        self.wait_expires = self.wait_since + datetime.timedelta(seconds = wait)

    def delete(self):
        '''
        @summary: explicit destructor
        '''
        if self.__delete_callback:
            (self.__delete_callback)()
        self.reset_callbacks()

    @classmethod
    def from_json(self, resource_name, json_string):
        '''
        @summary: class method which build a lock object from the corresponding json string
        @param resource_name: name of the resource
        @param json_string: json string
        @result: lock object (not active) or None if unseriliaze errors
        '''
        try:
            tmp = json.loads(json_string)
            title = tmp['title']
            wait = int(tmp['wait'])
            lifetime = int(tmp['lifetime'])
        except (KeyError, ValueError):
            return None
        return Lock(resource_name, title, wait, lifetime)

    def to_json(self):
        '''
        @summary: method which dumps the lock object as a json string
        @result: json string
        '''
        tmp = {"title": self.title,
               "wait": self.wait,
               "lifetime": self.lifetime}
        return json.dumps(tmp, indent=4, sort_keys=True)

    def set_active(self):
        '''
        @summary: change the status of the lock to active (corresponding callback is invoked)
        '''
        self.wait_since = None
        self.wait_expires = None
        self.__active = True
        self.active_since = datetime.datetime.now()
        self.active_expires = self.active_since + datetime.timedelta(seconds = self.lifetime)
        if self.__active_callback:
            (self.__active_callback)()
        self.reset_callbacks()
        
    def set_callbacks(self, active_callback, delete_callback):
        '''
        @summary: setter for callbacks
        @param active_callback: active callback
        @param delete_callback: delete callback
        '''
        self.__active_callback = active_callback
        self.__delete_callback = delete_callback

    def reset_callbacks(self):
        '''
        @summary: reset the lock callbacks
        '''
        self.__active_callback = None
        self.__delete_callback = None

    def is_expired(self):
        '''
        @summary: returns True is the lock is expired (wait timeout or lifetime timeout depending on the status)
        @result: True (the lock is expired) or False
        '''
        if self.__active:
            return (datetime.datetime.now() > self.active_expires)
        else:
            return (datetime.datetime.now() > self.wait_expires)


class Resource(object):
    """Class which defines a Resource object"""

    name = None
    active_lock = None
    __waiter_locks = None

    def __init__(self, name):
        '''
        @summary: constructor
        @param name: name of the resource
        '''
        self.name = name
        self.active_lock = None
        self.__waiter_locks = collections.deque()

    def remove_active_lock(self):
        '''
        @summary: remove the active lock of the resource (if any)
        
        Of course, if there is some non expired waiting locks,
        the first one is promoted as the active lock of the resource
        '''
        if self.active_lock:
            self.active_lock.delete()     
            self.active_lock = None
            while True:
                try:
                    lock = self.__waiter_locks.popleft()
                    if lock.is_expired():
                        lock.delete()
                        continue
                    lock.set_active()
                    self.active_lock = lock
                    break
                except IndexError:
                    break

    def add_lock(self, lock):
        '''
        @summary: add a lock to the resource
        @param lock: lock object
        
        If there is no active lock, the lock is promoted as the active
        lock for the resource

        If there is alreay an active lock, the lock is added to the 
        waiting list
        '''
        
        if not(self.active_lock):
            lock.set_active()
            self.active_lock = lock
        else:
            self.__waiter_locks.append(lock)

    def clean_expired_locks(self):
        '''
        @summary: clean expired lock of the resource (active and waiting)
        '''
        tmp_queue = collections.deque()
        while True:
            try:
                lock = self.__waiter_locks.popleft()
                if lock.is_expired():
                    lock.delete()
                    continue
                tmp_queue.append(lock)
            except IndexError:
                break
        self.__waiter_locks = tmp_queue
        if self.active_lock and self.active_lock.is_expired():
            self.remove_active_lock()


class LockManager(object):

    __resources_dict = {}

    def _reinit(self):
        '''
        @summary: reinitialize the lock manager (for unit testing only) 
        '''
        self.__resources_dict = {}

    def add_lock(self, resource_name, lock):
        '''
        @summary: add a lock to the resource
        @param resource_name: name of the resource
        @param lock: lock object
        
        If there is no active lock, the lock is promoted as the active
        lock for the resource

        If there is alreay an active lock, the lock is added to the 
        waiting list
        '''
        if not(self.__resources_dict.has_key(resource_name)):
            self.__resources_dict[resource_name] = Resource(resource_name)
        resource = self.__resources_dict[resource_name]
        resource.add_lock(lock)

    def remove_active_lock(self, resource_name):
        '''
        @summary: remove the active lock of the resource (if any)
        @param resource_name: name of the resource

        Of course, if there is some non expired waiting locks,
        the first one is promoted as the active lock of the resource
        '''
        if not(self.__resources_dict.has_key(resource_name)):
            self.__resources_dict[resource_name] = Resource(resource_name)
        resource = self.__resources_dict[resource_name]
        resource.remove_active_lock()

    def get_active_lock(self, resource_name):
        '''
        @summary: return the active lock for the resource name
        @param resource_name: name of the resource
        @result: active lock object (or None)
        '''
        if not(self.__resources_dict.has_key(resource_name)):
            self.__resources_dict[resource_name] = Resource(resource_name)
        resource = self.__resources_dict[resource_name]
        return resource.active_lock

    def clean_expired_locks(self):
        '''
        @summary: clean expired lock of all resources (active and waiting)
        '''
        names = self.__resources_dict.keys()
        for name in names:
            self.__resources_dict[name].clean_expired_locks()


LOCK_MANAGER_INSTANCE = LockManager()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.abstract_lock_handler import AbstractLockHandler
from rdlm.lock import Lock
import tornado.web
import tornado.gen
import tornado.ioloop
from rdlm.lock import LOCK_MANAGER_INSTANCE
import functools

class ActiveLocksHandler(AbstractLockHandler):
    """Class which handles the /active_locks/[...]/ URL"""

    def on_active_wrapper(self, name, lock):
        '''
        @summary: wrapper method to invoke on_active method through tornado ioloop
        @param name: name of the resource
        @param lock: lock object
        '''
        tornado.ioloop.IOLoop.instance().add_callback(functools.partial(self.on_active, name, lock))

    def on_delete_wrapper(self):
        '''
        @summary: wrapper method to invoke on_delete method through tornado ioloop 
        '''
        tornado.ioloop.IOLoop.instance().add_callback(self.on_delete)

    def on_active(self, name, lock):
        '''
        @summary: method called when the lock becomes active
        @param name: name of the resource
        @param lock: lock object

        The method returns an HTTP/201 in this case with 
        the corresponding Location header
        '''
        self.set_status(201)
        self.set_header("Location", "%s%s" % (self.get_base_url(self.request), self.reverse_url("lock", name, lock.uid)))
        self.finish()

    def on_delete(self):
        '''
        @summary: method called when the wait for the lock is over
        
        The method returns an HTTP/408 in this case
        '''
        self.send_error(status_code=408)

    @tornado.web.asynchronous
    def post(self, name): # pylint: disable-msg=W0221
        '''
        @summary: deals with POST request (acquiring locks on resource) 
        @param name: name of the resource
        '''
        raw_body = self.request.body
        if len(raw_body) == 0:
            self.send_error(status_code=400, message="empty body")
            return
        lock = Lock.from_json(name, raw_body)
        if not(lock):
            self.send_error(status_code=400, message="invalid json body")
            return
        lock.set_callbacks(functools.partial(self.on_active_wrapper, name, lock), self.on_delete_wrapper)
        LOCK_MANAGER_INSTANCE.add_lock(name, lock)
            
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

import tornado.web
import traceback
import httplib

class RequestHandler(tornado.web.RequestHandler):

    def write_error(self, status_code, **kwargs):
        if self.settings.get("debug") and "exc_info" in kwargs:
            # in debug mode, try to send a traceback
            self.set_header('Content-Type', 'text/plain')
            for line in traceback.format_exception(*kwargs["exc_info"]):
                self.write(line)
            self.finish()
        else:
            if kwargs.has_key('message'):
                message = kwargs['message']
            else:
                message = httplib.responses[status_code]
            self.finish("<html><title>%(code)d: %(message)s</title>"
                        "<body>%(code)d: %(message)s</body></html>" % {
                        "code": status_code,
                        "message": message,
                        })



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import tornado.testing
from rdlm.main import get_app as rdlm_get_app
from rdlm.main import get_ioloop as rdlm_get_ioloop
import json
import tornado.ioloop
import tornado.gen

TEST_MULTIPLE_WAITERS1 = 0
TEST_MULTIPLE_WAITERS2 = 0

class LockTestCase(tornado.testing.AsyncHTTPTestCase):

    def get_app(self):
        return rdlm_get_app(unit_testing=True)

    def get_new_ioloop(self): 
        return rdlm_get_ioloop()

    def tearDown(self):
        req = tornado.httpclient.HTTPRequest(self.get_url("/reset"))
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 200)
        super(LockTestCase, self).tearDown()

    def test_not_acquired_lock_bad_json(self):
        tmp = {"wait": 5, "lifetime": 10, "title": "test case"}
        raw_body = json.dumps(tmp)[0:10]
        req = tornado.httpclient.HTTPRequest(self.get_url('/active_locks/resource1'), method='POST', body=raw_body)
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 400)

    def test_not_acquired_lock_empty_body(self):
        raw_body = ""
        req = tornado.httpclient.HTTPRequest(self.get_url('/active_locks/resource1'), method='POST', body=raw_body)
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 400)

    def test_not_acquired_lock_missing_field(self):
        tmp = {"wait": 5, "lifetime": 10}
        raw_body = json.dumps(tmp)
        req = tornado.httpclient.HTTPRequest(self.get_url('/active_locks/resource1'), method='POST', body=raw_body)
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 400)

    def _acquire_lock(self, resource, wait, lifetime, title, callback=None):
        tmp = {"wait": wait, "lifetime": lifetime, "title": title}
        raw_body = json.dumps(tmp)
        req = tornado.httpclient.HTTPRequest(self.get_url('/active_locks/%s' % resource), method='POST', body=raw_body)
        if not(callback):
            self.http_client.fetch(req, self.stop)
            response = self.wait()
            self.assertEqual(response.code, 201)
            location = response.headers['Location']
            self.assertTrue(location.startswith('http://'))
            self.assertIn('/%s/' % resource, location)
            return location
        else:
            self.http_client.fetch(req, callback)
            
    def _delete_lock(self, lock_url):
        req = tornado.httpclient.HTTPRequest(lock_url, method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 204)

    def test_acquire_lock(self):
        self._acquire_lock("resource1", 5, 60, "test case")

    def test_delete_existing_lock(self):
        location = self._acquire_lock("resource1", 5, 60, "test case")
        req = tornado.httpclient.HTTPRequest(location, method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 204)

    def test_delete_non_existing_lock(self):
        location = self._acquire_lock("resource1", 5, 60, "test case")
        req = tornado.httpclient.HTTPRequest(location, method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 204)
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 404)
        location2 = location.replace('/resource1/', '/resource2/')
        req = tornado.httpclient.HTTPRequest(location2, method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 404)

    def test_get_existing_lock(self):
        location = self._acquire_lock("resource1", 5, 60, "test case")
        req = tornado.httpclient.HTTPRequest(location, method='GET')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 200)
        self.assertEqual(response.headers['Content-Type'], 'application/json')
        tmp = json.loads(response.body)
        self.assertEqual(tmp['title'], "test case")
        self.assertEqual(tmp['wait'], 5)
        self.assertEqual(tmp['lifetime'], 60)

    def test_get_not_existing_lock(self):
        location = self._acquire_lock("resource1", 5, 60, "test case")
        req = tornado.httpclient.HTTPRequest(location, method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 204)
        req = tornado.httpclient.HTTPRequest(location, method='GET')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 404)

    def test_expired_lock(self):
        location1 = self._acquire_lock("resource1", 5, 1, "test case")
        location2 = self._acquire_lock("resource1", 5, 60, "test case")
        req = tornado.httpclient.HTTPRequest(location1, method='GET')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 404)
        req = tornado.httpclient.HTTPRequest(location2, method='GET')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 200)

    def test_wait_timeout_lock(self):
        self._acquire_lock("resource1", 5, 60, "test case")
        tmp = {"wait": 1, "lifetime": 60, "title": "test case"}
        raw_body = json.dumps(tmp)
        req = tornado.httpclient.HTTPRequest(self.get_url('/active_locks/%s' % "resource1"), method='POST', body=raw_body)
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 408)

    def _test_multiple_waiters_callback2(self, r):
        self.assertEqual(r.code, 204)

    def _test_multiple_waiters_callback1(self, r):
        global TEST_MULTIPLE_WAITERS1 # pylint: disable-msg=W0603
        TEST_MULTIPLE_WAITERS1 = TEST_MULTIPLE_WAITERS1 + 1
        self.assertEqual(r.code, 201)
        lock_url = r.headers['Location']
        req = tornado.httpclient.HTTPRequest(lock_url, method='DELETE')
        self.http_client.fetch(req, callback=self._test_multiple_waiters_callback2)
        if TEST_MULTIPLE_WAITERS1 == 4:
            self.stop()

    def _test_multiple_waiters_callback3(self, r):
        global TEST_MULTIPLE_WAITERS2 # pylint: disable-msg=W0603
        TEST_MULTIPLE_WAITERS2 = TEST_MULTIPLE_WAITERS2 + 1
        self.assertEqual(r.code, 201)
        lock_url = r.headers['Location']
        req = tornado.httpclient.HTTPRequest(lock_url, method='DELETE')
        self.http_client.fetch(req, callback=self._test_multiple_waiters_callback2)
        if TEST_MULTIPLE_WAITERS2 == 4:
            self.stop()

    def test_multiple_waiters1(self):
        self._acquire_lock("resource1", 10, 60, "test case", callback=self._test_multiple_waiters_callback1)
        self._acquire_lock("resource2", 10, 60, "test case", callback=self._test_multiple_waiters_callback1)
        self._acquire_lock("resource3", 10, 60, "test case", callback=self._test_multiple_waiters_callback1)
        self._acquire_lock("resource4", 10, 60, "test case", callback=self._test_multiple_waiters_callback1)
        self.wait()

    def test_multiple_waiters2(self):
        self._acquire_lock("resource1", 10, 60, "test case", callback=self._test_multiple_waiters_callback3)
        self._acquire_lock("resource1", 10, 60, "test case", callback=self._test_multiple_waiters_callback3)
        self._acquire_lock("resource1", 10, 60, "test case", callback=self._test_multiple_waiters_callback3)
        self._acquire_lock("resource1", 10, 60, "test case", callback=self._test_multiple_waiters_callback3)
        self.wait()

        
        







    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from distutils.core import setup

description = "RDLM (Restful Distributed Lock Manager) is a lock manager over HTTP build on Tornado"
try:
    with open('README.rst') as f:
        long_description = f.read()
except IOError:
    long_description = description

setup(
    name='rdlm',
    version='0.1a1',
    author="Fabien MARTY",
    author_email="fabien.marty@gmail.com",
    url="https://github.com/thefab/restful-distributed-lock-manager",
    packages=['rdlm',],
    license='MIT',
    download_url='https://github.com/thefab/restful-distributed-lock-manager',
    description=description,
    long_description=long_description,
    scripts=["rdlm-daemon.py"],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',    
        'Topic :: Utilities',
        'Topic :: System :: Distributed Computing',
        'Topic :: Software Development',
        'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content'
      ]
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from multiprocessing import Pool
import requests

PROCESS_POOL_SIZE = 10
REQUESTS = 10000
BASE_URL = "http://localhost:8888"
RESOURCE_NAME_PREFIX = "resource"

def f(process_number):
    resource_name = "%s%i" % (RESOURCE_NAME_PREFIX, process_number)
    raw_body = '{"title": "%i", "lifetime": 300, "wait": 20}' % process_number
    r = requests.post("%s/active_locks/%s" % (BASE_URL, resource_name), data=raw_body)
    if r.status_code != 201:
        raise Exception("bad status code %i from post request" % r.status_code)
    lock_url = r.headers['Location']
    r = requests.delete(lock_url)
    if r.status_code != 204:
        raise Exception("bad status code %i from delete request" % r.status_code)
 
if __name__ == '__main__':
    pool = Pool(processes=PROCESS_POOL_SIZE)
    pool.map(f, range(0, REQUESTS))
